//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102
//     EMail:  loreti@pd.infn.it
//     WWW:    http://www.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------------
// Implementation of the procedure declared in "generr.hh".
// According to the preprocessor symbol "GENERR_DATETIME", the message
// contains:
// GENERR_DATETIME = 1 : current time
// GENERR_DATETIME = 2 : current date and time
// GENERR_DATETIME = none of the above values : nothing
// -------------------------------------------------------------------
// $Id: generr.cxx,v 1.4 2005/06/07 06:03:33 loreti Exp $
// -------------------------------------------------------------------

#include <ctime>
#include "generr.hh"

using namespace std;

namespace {
  const char t_fmt[] = "%H:%M:%S";
  const char d_fmt[] = "%d-%b-%Y";
  const char a_fmt[] = "%d-%b-%Y %H:%M:%S";
}

namespace generr {
  void errmes(
    const char * fileName,
    unsigned int lineNumber,
    const char * severity,
    const char * message,
    ostream &    o              // Defaults to cerr
  ) {

#if GENERR_DATETIME == 1 || GENERR_DATETIME == 2
    time_t      now = time(0);
    struct tm * ptm = localtime(&now);
#if GENERR_DATETIME == 1
    char fmtNow[] = "HH:MM:SS";
    strftime(fmtNow, sizeof(fmtNow), t_fmt, ptm);
#else  // == 2
    char fmtNow[] = "DD-MMM-YYYY HH:MM:SS";
    strftime(fmtNow, sizeof(fmtNow), a_fmt, ptm);
#endif
    o << "* " << fmtNow << ' ';
#endif

    o << "* " << severity << " from file " << fileName
      << " at line "  << lineNumber << "\n* Message: "
      << message << endl;
  }
}

void genex::datim() {
  time_t      now = time(0);
  struct tm * ptm = localtime(&now);
  strftime(_date, sizeof(_date), d_fmt, ptm);
  strftime(_time, sizeof(_time), t_fmt, ptm);
}
