//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102
//     EMail:  loreti@pd.infn.it
//     WWW:    http://www.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------------
// General error printout routines
// -------------------------------------------------------------------
// $Id: generr.hh,v 1.4 2005/06/07 06:03:33 loreti Exp $
// -------------------------------------------------------------------

#ifndef GENERR_HH_
#define GENERR_HH_

#include <iostream>
#include <stdexcept>
#include <string>

namespace generr {
  template <typename T> void valnam(
    const char *   name,
    const T &      value,
    std::ostream & o      = std::cerr
  ) {
    // Prints on the output stream "o" the name of a variable and its
    // value.  The variable must have a valid "operator <<" defined.

    o << "  -> " << name << " = " << value << std::endl;
  }

  // Declares a procedure for the printout of generic error.  The
  // arguments are: file name; line number; severity level; error
  // message; and the output stream.

  void errmes(const char *, unsigned int, const char *,
              const char *, std::ostream & = std::cerr);
}

// Preprocessor macros to ease the use of the above procedures.

#define VALNAM(x)  generr::valnam(#x, x)

#define ERRMES(m)  generr::errmes(__FILE__, __LINE__, "Error",      m)
#define WARNMES(m) generr::errmes(__FILE__, __LINE__, "Warning",    m)
#define INFOMES(m) generr::errmes(__FILE__, __LINE__, "Info",       m)
#define DBGMES(m)  generr::errmes(__FILE__, __LINE__, "Debug info", m)

// A class derived from std::exception, that saves for the catcher
// informations about date, time, file name and line that may be
// retrieved and, if needed, printed.

class genex : public std::runtime_error {
private:
  char         _date[12];
  char         _time[9];
  std::string  _file;
  unsigned int _line;

  void datim();

public:
  genex(const char * w, const char * f, unsigned int l)
    : std::runtime_error(w), _file(f), _line(l) { datim(); }
  ~genex() throw() {}

  const char *        getDate() const { return _date; }
  const char *        getTime() const { return _time; }
  const std::string & getFile() const { return _file; }
  const unsigned int  getLine() const { return _line; }
};

// Preprocessor macros to ease the use of the above class

#define GENEX(w) throw genex(w, __FILE__, __LINE__)

#endif // GENERR_HH_
